package com.persagy.init;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.boot.CommandLineRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.util.ResourceUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.persagy.constant.CommonConst;
import com.persagy.constant.ExtraCommonConstant;
import com.persagy.entity.ReturnAlarmDefine;

import cn.hutool.core.util.BooleanUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 项目启动后，加载控返不一致报警所需数据
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年10月14日 下午5:09:18
 */
@Slf4j
@Order(-10)
@Component
public class InitReturnAlarmConfig implements CommandLineRunner {
	
	/** 是否开启控返不一致的报警 */
	public static volatile boolean alarmEnable = false;
	
	/** key: meter-funcid（控制信息点）, value: 报警定义  */
	public static final Map<String, ReturnAlarmDefine> controlCode2AlarmDefine = new ConcurrentHashMap<String, ReturnAlarmDefine>();
	/** key: meter-funcid（反馈信息点），value:报警定义 */
	public static final Map<String, ReturnAlarmDefine> returnCode2AlarmDefine = new ConcurrentHashMap<String, ReturnAlarmDefine>();
	
	/** key：meter-funcid（报警信息点）, value：1-报警，0-正常，-1-不作任何处理 */
	public static final Map<String, Integer> ibmsCode2AlarmResult = new ConcurrentHashMap<String, Integer>();
	/** key：meter-funcid（报警信息点）, value：报警信息点数据 */
	public static final Map<String, JSONObject> ibmsCode2AlarmData = new ConcurrentHashMap<String, JSONObject>();
	/** key：meter-funcid（报警信息点）, value: 报警个数 */
	public static final Map<String, Integer> alarmCount = new ConcurrentHashMap<String, Integer>();
	
	@Override
	public void run(String... args) throws Exception {
		if (!CommonConst.collectEnable) {
			return;
		}
		// 获取控返不一致报警配置文件
		File file = this.getFile();
		if (file == null || !file.exists()) {
			return;
		}
		// 解析文件
		this.parseAlarmConfig(file);
		log.info("控返不一致报警配置文件加载完成");
	}
	
	/**
	 * 文件加载，并放入本地缓存
	 * 
	 * @param alarmFile
	 * @throws IOException 
	 * @date 2021年10月14日 下午5:35:03
	 */
	private void parseAlarmConfig(File alarmFile) throws IOException {
		JSONObject fileJSON = this.readFileInfo(alarmFile);
		alarmEnable = fileJSON.getBoolean("alarm-enable");
		if (!BooleanUtil.isTrue(alarmEnable)) {
			return;
		}
		
		JSONArray alarmConfigs = fileJSON.getJSONArray("alarm-config");
		for (int i = 0; i < alarmConfigs.size(); i++) {
			JSONObject alarmConfig = alarmConfigs.getJSONObject(i);
			// 获取报警的设备类配置
			String ibmsSceneCode = alarmConfig.getString("ibmsSceneCode");
			String ibmsClassCode = alarmConfig.getString("ibmsClassCode");
			JSONArray alarmDefines = alarmConfig.getJSONArray("alarmDefine");
			
			// 报警定义配置
			JSONArray objects = ExtraCommonConstant.getObjectArray(ibmsSceneCode, ibmsClassCode);
			if (objects != null) {
				ExtraCommonConstant.getAlarmDefines(ibmsSceneCode, ibmsClassCode, alarmDefines, objects);
			}
		}
	}
	
	/**
	 * 读取文件内容，返回文件信息
	 * 
	 * @return
	 * @throws IOException 
	 * @date 2021年10月14日 下午5:37:29
	 */
	private JSONObject readFileInfo(File file) throws IOException {
		return JSON.parseObject(new FileInputStream(file), StandardCharsets.UTF_8, JSONObject.class,
                // 自动关闭流
                Feature.AutoCloseSource,
                // 允许注释
                Feature.AllowComment,
                // 允许单引号
                Feature.AllowSingleQuotes);
		
	}
	
	
	private File getFile() throws FileNotFoundException {
		File file = null;
		try {
			// 先获取config目录下的文件，获取不到，再获取classpath下的
			file = ResourceUtils.getFile(CommonConst.SERVER_ROOT_PATH + "/config/return-alarm.json");
			if (!file.exists()) {
				log.warn("config目录下不存在控返不一致的报警配置，获取classpath下的默认配置");
				file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "return-alarm.json");
			}
		} catch (Exception e) {
		}
		return file;
	}
	
}
